### Prisma follows the same constants as the curses module.
ERR = -1
OK = 0
A_ATTRIBUTES = 4294967040
A_NORMAL = 0
A_STANDOUT = 65536
A_UNDERLINE = 131072
A_REVERSE = 262144
A_BLINK = 524288
A_DIM = 1048576
A_BOLD = 2097152
A_ALTCHARSET = 4194304
A_INVIS = 8388608
A_PROTECT = 16777216
A_CHARTEXT = 255
A_COLOR = 65280
A_HORIZONTAL = 33554432
A_LEFT = 67108864
A_LOW = 134217728
A_RIGHT = 268435456
A_TOP = 536870912
A_VERTICAL = 1073741824
A_ITALIC = 2147483648
COLOR_BLACK = 0
COLOR_RED = 1
COLOR_GREEN = 2
COLOR_YELLOW = 3
COLOR_BLUE = 4
COLOR_MAGENTA = 5
COLOR_CYAN = 6
COLOR_WHITE = 7
BUTTON1_PRESSED = 2
BUTTON1_RELEASED = 1
BUTTON1_CLICKED = 4
BUTTON1_DOUBLE_CLICKED = 8
BUTTON1_TRIPLE_CLICKED = 16
BUTTON2_PRESSED = 64
BUTTON2_RELEASED = 32
BUTTON2_CLICKED = 128
BUTTON2_DOUBLE_CLICKED = 256
BUTTON2_TRIPLE_CLICKED = 512
BUTTON3_PRESSED = 2048
BUTTON3_RELEASED = 1024
BUTTON3_CLICKED = 4096
BUTTON3_DOUBLE_CLICKED = 8192
BUTTON3_TRIPLE_CLICKED = 16384
BUTTON4_PRESSED = 65536
BUTTON4_RELEASED = 32768
BUTTON4_CLICKED = 131072
BUTTON4_DOUBLE_CLICKED = 262144
BUTTON4_TRIPLE_CLICKED = 524288
BUTTON5_PRESSED = 2097152
BUTTON5_RELEASED = 1048576
BUTTON5_CLICKED = 4194304
BUTTON5_DOUBLE_CLICKED = 8388608
BUTTON5_TRIPLE_CLICKED = 16777216
BUTTON_SHIFT = 67108864
BUTTON_CTRL = 33554432
BUTTON_ALT = 134217728
ALL_MOUSE_EVENTS = 268435455
REPORT_MOUSE_POSITION = 268435456
KEY_BREAK = 257
KEY_DOWN = 258
KEY_UP = 259
KEY_LEFT = 260
KEY_RIGHT = 261
KEY_HOME = 262
KEY_BACKSPACE = 263
KEY_F0 = 264
KEY_F1 = 265
KEY_F2 = 266
KEY_F3 = 267
KEY_F4 = 268
KEY_F5 = 269
KEY_F6 = 270
KEY_F7 = 271
KEY_F8 = 272
KEY_F9 = 273
KEY_F10 = 274
KEY_F11 = 275
KEY_F12 = 276
KEY_F13 = 277
KEY_F14 = 278
KEY_F15 = 279
KEY_F16 = 280
KEY_F17 = 281
KEY_F18 = 282
KEY_F19 = 283
KEY_F20 = 284
KEY_F21 = 285
KEY_F22 = 286
KEY_F23 = 287
KEY_F24 = 288
KEY_F25 = 289
KEY_F26 = 290
KEY_F27 = 291
KEY_F28 = 292
KEY_F29 = 293
KEY_F30 = 294
KEY_F31 = 295
KEY_F32 = 296
KEY_F33 = 297
KEY_F34 = 298
KEY_F35 = 299
KEY_F36 = 300
KEY_F37 = 301
KEY_F38 = 302
KEY_F39 = 303
KEY_F40 = 304
KEY_F41 = 305
KEY_F42 = 306
KEY_F43 = 307
KEY_F44 = 308
KEY_F45 = 309
KEY_F46 = 310
KEY_F47 = 311
KEY_F48 = 312
KEY_F49 = 313
KEY_F50 = 314
KEY_F51 = 315
KEY_F52 = 316
KEY_F53 = 317
KEY_F54 = 318
KEY_F55 = 319
KEY_F56 = 320
KEY_F57 = 321
KEY_F58 = 322
KEY_F59 = 323
KEY_F60 = 324
KEY_F61 = 325
KEY_F62 = 326
KEY_F63 = 327
KEY_DL = 328
KEY_IL = 329
KEY_DC = 330
KEY_IC = 331
KEY_EIC = 332
KEY_CLEAR = 333
KEY_EOS = 334
KEY_EOL = 335
KEY_SF = 336
KEY_SR = 337
KEY_NPAGE = 338
KEY_PPAGE = 339
KEY_STAB = 340
KEY_CTAB = 341
KEY_CATAB = 342
KEY_ENTER = 343
KEY_SRESET = 344
KEY_RESET = 345
KEY_PRINT = 346
KEY_LL = 347
KEY_A1 = 348
KEY_A3 = 349
KEY_B2 = 350
KEY_C1 = 351
KEY_C3 = 352
KEY_BTAB = 353
KEY_BEG = 354
KEY_CANCEL = 355
KEY_CLOSE = 356
KEY_COMMAND = 357
KEY_COPY = 358
KEY_CREATE = 359
KEY_END = 360
KEY_EXIT = 361
KEY_FIND = 362
KEY_HELP = 363
KEY_MARK = 364
KEY_MESSAGE = 365
KEY_MOVE = 366
KEY_NEXT = 367
KEY_OPEN = 368
KEY_OPTIONS = 369
KEY_PREVIOUS = 370
KEY_REDO = 371
KEY_REFERENCE = 372
KEY_REFRESH = 373
KEY_REPLACE = 374
KEY_RESTART = 375
KEY_RESUME = 376
KEY_SAVE = 377
KEY_SBEG = 378
KEY_SCANCEL = 379
KEY_SCOMMAND = 380
KEY_SCOPY = 381
KEY_SCREATE = 382
KEY_SDC = 383
KEY_SDL = 384
KEY_SELECT = 385
KEY_SEND = 386
KEY_SEOL = 387
KEY_SEXIT = 388
KEY_SFIND = 389
KEY_SHELP = 390
KEY_SHOME = 391
KEY_SIC = 392
KEY_SLEFT = 393
KEY_SMESSAGE = 394
KEY_SMOVE = 395
KEY_SNEXT = 396
KEY_SOPTIONS = 397
KEY_SPREVIOUS = 398
KEY_SPRINT = 399
KEY_SREDO = 400
KEY_SREPLACE = 401
KEY_SRIGHT = 402
KEY_SRSUME = 403
KEY_SSAVE = 404
KEY_SSUSPEND = 405
KEY_SUNDO = 406
KEY_SUSPEND = 407
KEY_UNDO = 408
KEY_MOUSE = 409
KEY_RESIZE = 410
KEY_MIN = 257
KEY_MAX = 511

### Additionally, ASCII characters (32-126) are provided.
KEY_SPACE  = 32
KEY_EXCLAMATION = 33
KEY_QUOTATION = 34
KEY_NUMBER = 35
KEY_DOLLAR = 36
KEY_PERCENT = 37
KEY_AMPERSAND = 38
KEY_APOSTROPHE = 39
KEY_L_PARENTHESIS = 40
KEY_R_PARENTHESIS = 41
KEY_ATERISK = 42
KEY_PLUS = 43
KEY_COMMA = 44
KEY_HYPHEN = 45
KEY_PERIOD = 46
KEY_SLASH = 47
KEY_0 = 48
KEY_1 = 49
KEY_2 = 50
KEY_3 = 51
KEY_4 = 52
KEY_5 = 53
KEY_6 = 54
KEY_7 = 55
KEY_8 = 56
KEY_9 = 57
KEY_COLON = 58
KEY_SEMICOLON = 59
KEY_LESS_THAN = 60
KEY_EQUAL = 61
KEY_GREATER_THAN = 62
KEY_QUESTION = 63
KEY_AT = 64
KEY_A_UPPER = 65
KEY_B_UPPER = 66
KEY_C_UPPER = 67
KEY_D_UPPER = 68
KEY_E_UPPER = 69
KEY_F_UPPER = 70
KEY_G_UPPER = 71
KEY_H_UPPER = 72
KEY_I_UPPER = 73
KEY_J_UPPER = 74
KEY_K_UPPER = 75
KEY_L_UPPER = 76
KEY_M_UPPER = 77
KEY_N_UPPER = 78
KEY_O_UPPER = 79
KEY_P_UPPER = 80
KEY_Q_UPPER = 81
KEY_R_UPPER = 82
KEY_S_UPPER = 83
KEY_T_UPPER = 84
KEY_U_UPPER = 85
KEY_V_UPPER = 86
KEY_W_UPPER = 87
KEY_X_UPPER = 88
KEY_Y_UPPER = 89
KEY_Z_UPPER = 90
KEY_L_SQUARE_BRACKET = 91
KEY_BACKSLASH = 92
KEY_R_SQUARE_BRACKET = 93
KEY_CARET = 94
KEY_UNDERSCORE = 95
KEY_GRAVE = 96
KEY_A_LOWER = 97
KEY_B_LOWER = 98
KEY_C_LOWER = 99
KEY_D_LOWER = 100
KEY_E_LOWER = 101
KEY_F_LOWER = 102
KEY_G_LOWER = 103
KEY_H_LOWER = 104
KEY_I_LOWER = 105
KEY_J_LOWER = 106
KEY_K_LOWER = 107
KEY_L_LOWER = 108
KEY_M_LOWER = 109
KEY_N_LOWER = 110
KEY_O_LOWER = 111
KEY_P_LOWER = 112
KEY_Q_LOWER = 113
KEY_R_LOWER = 114
KEY_S_LOWER = 115
KEY_T_LOWER = 116
KEY_U_LOWER = 117
KEY_V_LOWER = 118
KEY_W_LOWER = 119
KEY_X_LOWER = 120
KEY_Y_LOWER = 121
KEY_Z_LOWER = 122
KEY_L_CURLY_BRACE = 123
KEY_VERTICAL_BAR = 124
KEY_L_CURLY_BRACE = 125
KEY_TILDE = 126
